/**
 * @copyright  (C) 2022 Xiroweb
 * @copyright  (C) 2018 Open Source Matters, Inc. <https://www.joomla.org>
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

/**
 * 'Showvalueclass' class
 */
class Showvalueclass {
  /**
   * Constructor
   *
   * @param {HTMLElement} cont Container element
   */
  constructor(cont) {
    const self = this;
    this.container = cont || document;
    this.fields = {// origin-field-name: {
      //   origin:  ['collection of all the trigger nodes'],
      //   targets: ['collection of nodes to be controlled control']
      // }
    };
    this.showonFields = [].slice.call(this.container.querySelectorAll('[data-showvalueclass]')); // Populate the fields data

    if (this.showonFields.length) {
      // @todo refactor this, dry
      this.showonFields.forEach(field => {
        // Set up only once
        if (field.hasAttribute('data-showvalueclass-initialised')) {
          return;
        }

        field.setAttribute('data-showvalueclass-initialised', '');
        const jsondata = field.getAttribute('data-showvalueclass') || '';
        const showonData = JSON.parse(jsondata);
        let localFields;

        if (showonData.length) {
          localFields = [].slice.call(self.container.querySelectorAll(`[name="${showonData[0].field}"], [name="${showonData[0].field}[]"]`));

          if (!this.fields[showonData[0].field]) {
            this.fields[showonData[0].field] = {
              origin: [],
              targets: []
            };
          } // Add trigger elements


          localFields.forEach(cField => {
            if (this.fields[showonData[0].field].origin.indexOf(cField) === -1) {
              this.fields[showonData[0].field].origin.push(cField);
            }
          }); // Add target elements

          this.fields[showonData[0].field].targets.push(field); // Data showon can have multiple values

          if (showonData.length > 1) {
            showonData.forEach((value, index) => {
              if (index === 0) {
                return;
              }

              localFields = [].slice.call(self.container.querySelectorAll(`[name="${value.field}"], [name="${value.field}[]"]`));

              if (!this.fields[showonData[0].field]) {
                this.fields[showonData[0].field] = {
                  origin: [],
                  targets: []
                };
              } // Add trigger elements


              localFields.forEach(cField => {
                if (this.fields[showonData[0].field].origin.indexOf(cField) === -1) {
                  this.fields[showonData[0].field].origin.push(cField);
                }
              }); // Add target elements

              if (this.fields[showonData[0].field].targets.indexOf(field) === -1) {
                this.fields[showonData[0].field].targets.push(field);
              }
            });
          }
        }
      }); // Do some binding

      this.linkedOptions = this.linkedOptions.bind(this); // Attach events to referenced element, to check condition on change and keyup

      Object.keys(this.fields).forEach(key => {
        if (this.fields[key].origin.length) {
          this.fields[key].origin.forEach(elem => {
            // Initialize the showon behaviour for the given HTMLElement
            self.linkedOptions(key); // Setup listeners

            elem.addEventListener('change', () => {
              self.linkedOptions(key);
            });
            elem.addEventListener('keyup', () => {
              self.linkedOptions(key);
            });
            elem.addEventListener('click', () => {
              self.linkedOptions(key);
            });
          });
        }
      });
    }
  }
  /**
   *
   * @param key
   */


  linkedOptions(key) {
    // Loop through the elements that need to be either shown or hidden
    this.fields[key].targets.forEach(field => {
      const elementShowvalueclassDatas = JSON.parse(field.getAttribute('data-showvalueclass')) || [];
      let itemval; // Check if target conditions are satisfied

      let valuecond;
      elementShowvalueclassDatas.forEach((elementShowvalueclassData, index) => {
        const condition = elementShowvalueclassData || {};
        condition.valid = 0; // Test in each of the elements in the field array if condition is valid

        valuecond = elementShowvalueclassData.values[0];
        this.fields[key].origin.forEach(originField => {
          if (originField.name.replace('[]', '') !== elementShowvalueclassData.field) {
            return;
          }

          const originId = originField.id; // If checkbox or radio box the value is read from properties

          if (originField.getAttribute('type') && ['checkbox', 'radio'].includes(originField.getAttribute('type').toLowerCase())) {
            if (!originField.checked) {
              // Unchecked fields will return a blank and so always match
              // a != condition so we skip them
              return;
            }

            itemval = document.getElementById(originId).value;
          } else if (originField.nodeName === 'SELECT' && originField.hasAttribute('multiple')) {
            itemval = Array.from(originField.querySelectorAll('option:checked')).map(el => el.value);
          } else {
            // Select lists, text-area etc. Note that multiple-select list returns
            // an Array here s0 we can always treat 'itemval' as an array
            itemval = document.getElementById(originId).value; // A multi-select <select> $field  will return null when no elements are
            // selected so we need to define itemval accordingly

            if (itemval === null && originField.tagName.toLowerCase() === 'select') {
              itemval = [];
            }
          } // Convert to array to allow multiple values in the field (e.g. type=list multiple)
          // and normalize as string


          if (!(typeof itemval === 'object')) {
            itemval = JSON.parse(`["${itemval}"]`);
          } // Test if any of the values of the field exists in showon conditions

        }); // Verify conditions
        // First condition (no operator): current condition must be valid
      }); // If conditions are satisfied show the target field(s), else hide

      let gclassname = field.className; // remove class
      // const regex = new RegExp(`${valuecond}\\d+`, 'g')

      const regex = new RegExp(`(${valuecond})\\w+`, 'g');
      let gclassname2 = gclassname.replaceAll(regex, '');
      field.className = gclassname2; // add class

      field.classList.add(valuecond + itemval);
      field.dispatchEvent(new CustomEvent('joomla:showvalueclass-change', {
        bubbles: true
      }));
    });
  }

}

if (!window.Joomla) {
  throw new Error('Joomla API is not properly initialized');
} // Provide a public API


if (!Joomla.Showvalueclass) {
  Joomla.Showvalueclass = {
    initialise: container => new Showvalueclass(container)
  };
}
/**
 * Initialize 'showon' feature at an initial page load
 */


Joomla.Showvalueclass.initialise(document);
/**
 * Search for matching parents
 *
 * @param {HTMLElement} $child
 * @param {String} selector
 * @returns {HTMLElement[]}
 */

const getMatchedParents = ($child, selector) => {
  let $parent = $child;
  let $matchingParent;
  const parents = [];

  while ($parent) {
    $matchingParent = $parent.matches && $parent.matches(selector) ? $parent : null;

    if ($matchingParent) {
      parents.unshift($matchingParent);
    }

    $parent = $parent.parentNode;
  }

  return parents;
};
/**
 * Initialize 'showon' feature when part of the page was updated
 */


document.addEventListener('joomla:updated', ({
  target
}) => {
  // Check is it subform, then wee need to fix some "showon" config
  if (target.classList.contains('subform-repeatable-group')) {
    const elements = [].slice.call(target.querySelectorAll('[data-showvalueclass]'));

    if (elements.length) {
      const search = [];
      const replace = []; // Collect all parent groups of changed group

      getMatchedParents(target, '.subform-repeatable-group').forEach($parent => {
        search.push(new RegExp(`\\[${$parent.dataset.baseName}X\\]`, 'g'));
        replace.push(`[${$parent.dataset.group}]`);
      }); // Fix showon field names in a current group

      elements.forEach(element => {
        let {
          showon
        } = element.dataset;
        search.forEach((pattern, i) => {
          showon = showon.replace(pattern, replace[i]);
        });
        element.dataset.showon = showon;
      });
    }
  }

  Joomla.Showon.initialise(target);
});
